unit Utils;

interface

uses
  Windows, SysUtils, ifpscomp, ifps3;

procedure RegisterNSISLibrary_R(Exec: TIFPSExec);
procedure RegisterNSISLibrary_C(Comp: TIFPSPascalCompiler);
implementation
uses TypInfo, NSIS, Commctrl;

procedure Push(const S: String);
begin
  PushString(PChar(S));
end;

function Pop(var S: String): Boolean;
var
  P: PChar;
begin
  GetMem(P, g_stringsize);
  try
    Result := PopString(P) = 0;
    if Result then S := P;
  finally
    FreeMem(P);
  end;
end;

procedure SetUserVariable(varnum: Cardinal; value: String);
begin
  NSIS.SetUserVariable(TVariableList(varnum), PChar(Value));
end;

function GetUserVariable(varnum: Cardinal): String;
begin
  Result := NSIS.GetUserVariable(TVariableList(varnum));
end;

function GetStackTop: Pointer;
begin
  Result := g_stacktop;
end;

procedure DetailPrint(const S: String);
var
  hwndList, hwndDlg: HWND;
  nItemCount: Integer;
  item: TLVItem;
begin
  hwndDlg := FindWindowEx(g_hwndParent, 0, '#32770', nil);
  hwndList := FindWindowEx(hwndDlg, 0, 'SysListView32', nil);

  FillChar(item, SizeOf(item), 0);
  nItemCount := SendMessage(hwndList, LVM_GETITEMCOUNT, 0, 0);
  item.mask := LVIF_TEXT;
  item.pszText := PChar(S);
  item.cchTextMax := Length(S);
  item.iItem := nItemCount;
  ListView_InsertItem(hwndList, item);
  ListView_EnsureVisible(hwndList, item.iItem, LongBool(0));
end;

function MessageBox(const Text, Caption: String; uType: Cardinal): Integer;
begin
  Result := Windows.MessageBox(g_hwndParent, PChar(Text), PChar(Caption), uType);
end;

function CallInstDll(const DLLFileName, FuncName: String;
 const NoUnload: Boolean): Boolean;
var
  SaveErrorMode: DWord;
  DLLHandle: HINST;
  Func: procedure(hwndParent: HWND; string_size: integer; variables: PChar; stacktop: pointer);
begin
  Result := False;
  SaveErrorMode := SetErrorMode(SEM_NOOPENFILEERRORBOX or SEM_FAILCRITICALERRORS);
  try
    DLLHandle := GetModuleHandle(PChar(DLLFileName));
    if DLLHandle = 0 then
      DLLHandle := LoadLibrary(PChar(DLLFileName));
    if DLLHandle <> 0 then
    begin
      Func := GetProcAddress(DLLHandle, PChar(FuncName));
      if Assigned(Func) then
      begin
        Func(g_hwndParent, g_stringsize, g_variables, g_stacktop);
        Result := True;
      end;
      if not NoUnload then
        FreeLibrary(DLLHandle);
    end;
  finally
    SetErrorMode(SaveErrorMode);
  end;
end; 

procedure RegisterNSISLibrary_R(Exec: TIFPSExec);
begin
  Exec.RegisterDelphiFunction(@Push, 'PUSH', cdRegister);
  Exec.RegisterDelphiFunction(@Pop, 'POP', cdRegister);
  Exec.RegisterDelphiFunction(@SetUserVariable, 'SETUSERVARIABLE', cdRegister);
  Exec.RegisterDelphiFunction(@GetUserVariable, 'GETUSERVARIABLE', cdRegister);
  Exec.RegisterDelphiFunction(@MessageBox, 'MESSAGEBOX', cdRegister);
  Exec.RegisterDelphiFunction(@ParamStr, 'PARAMSTR', cdRegister);
  Exec.RegisterDelphiFunction(@DetailPrint, 'DETAILPRINT', cdRegister);
  Exec.RegisterDelphiFunction(@GetStackTop, 'GETSTACKTOP', cdRegister);
  Exec.RegisterDelphiFunction(@ParamCount, 'PARAMCOUNT', cdRegister);
  Exec.RegisterDelphiFunction(@CallInstDll, 'CALLINSTDLL', cdRegister);
end;


procedure RegisterNSISLibrary_C(Comp: TIFPSPascalCompiler);

  procedure RegisterConst(const Name: String; const Value: LongInt);
  var
    C: PIFPSConstant;
  begin
    C := Comp.AddConstant(UpperCase(Name), Comp.FindType('LONGINT'));
    C.Value.tU32 := Value;
  end;

  procedure RegisterEnum(const TypeInfo: PTypeInfo; const MinValue, MaxValue: Integer);
  var
    Value: Integer;
  begin
    for Value := MinValue to MaxValue do
      RegisterConst(GetEnumName(TypeInfo, Value), Value);
  end;


begin
  Comp.AddTypeS('stack_t', 'record next: Pointer; text: String; end');
  Comp.AddTypeS('TVariableList', 'Cardinal');
  RegisterConst('hwndParent', g_hwndParent);
  RegisterConst('StringSize', g_stringsize);
  RegisterEnum(TypeInfo(TVariableList), Ord(Low(TVariableList)), Ord(High(TVariableList)));
  
  RegisterConst('MB_OK', MB_OK);
  RegisterConst('MB_OKCANCEL', MB_OKCANCEL);
  RegisterConst('MB_ABORTRETRYIGNORE', MB_ABORTRETRYIGNORE);
  RegisterConst('MB_YESNOCANCEL', MB_YESNOCANCEL);
  RegisterConst('MB_YESNO', MB_YESNO);
  RegisterConst('MB_RETRYCANCEL', MB_RETRYCANCEL);
  RegisterConst('MB_DEFBUTTON1', MB_DEFBUTTON1);
  RegisterConst('MB_DEFBUTTON2', MB_DEFBUTTON2);
  RegisterConst('MB_DEFBUTTON3', MB_DEFBUTTON3);
  RegisterConst('MB_SETFOREGROUND', MB_SETFOREGROUND);

  RegisterConst('MB_ICONHAND', MB_ICONHAND);
  RegisterConst('MB_ICONQUESTION', MB_ICONQUESTION);
  RegisterConst('MB_ICONEXCLAMATION', MB_ICONEXCLAMATION);
  RegisterConst('MB_ICONASTERISK', MB_ICONASTERISK);
  RegisterConst('MB_USERICON', MB_USERICON);
  RegisterConst('MB_ICONWARNING', MB_ICONWARNING);
  RegisterConst('MB_ICONERROR', MB_ICONERROR);
  RegisterConst('MB_ICONINFORMATION', MB_ICONINFORMATION);
  RegisterConst('MB_ICONSTOP', MB_ICONSTOP);

  RegisterConst('IDOK', IDOK);
  RegisterConst('IDCANCEL', IDCANCEL);
  RegisterConst('IDABORT', IDABORT);
  RegisterConst('IDRETRY', IDRETRY);
  RegisterConst('IDIGNORE', IDIGNORE);
  RegisterConst('IDYES', IDYES);
  RegisterConst('IDNO', IDNO);

  RegisterConst('HKEY_CLASSES_ROOT', LongInt(HKEY_CLASSES_ROOT));
  RegisterConst('HKEY_CURRENT_USER', LongInt(HKEY_CURRENT_USER));
  RegisterConst('HKEY_LOCAL_MACHINE', LongInt(HKEY_LOCAL_MACHINE));
  RegisterConst('HKEY_USERS', LongInt(HKEY_USERS));
  RegisterConst('HKEY_PERFORMANCE_DATA', LongInt(HKEY_PERFORMANCE_DATA));
  RegisterConst('HKEY_CURRENT_CONFIG', LongInt(HKEY_CURRENT_CONFIG));
  RegisterConst('HKEY_DYN_DATA', LongInt(HKEY_DYN_DATA));

  RegisterConst('HKCR', LongInt(HKEY_CLASSES_ROOT));
  RegisterConst('HKCU', LongInt(HKEY_CURRENT_USER));
  RegisterConst('HKLM', LongInt(HKEY_LOCAL_MACHINE));
  RegisterConst('HKU', LongInt(HKEY_USERS));
  RegisterConst('HKCC', LongInt(HKEY_CURRENT_CONFIG));

  RegisterConst('SW_SHOWNORMAL', SW_SHOWNORMAL);
  RegisterConst('SW_SHOWMAXIMIZED', SW_SHOWMAXIMIZED);
  RegisterConst('SW_SHOWMINNOACTIVE', SW_SHOWMINNOACTIVE);
  RegisterConst('SW_HIDE', SW_HIDE);

  Comp.AddDelphiFunction('procedure Push(const S: String)');
  Comp.AddDelphiFunction('function Pop(var S: String): Boolean');
  Comp.AddDelphiFunction('function GetUserVariable(varnum: Cardinal): String');
  Comp.AddDelphiFunction('procedure SetUserVariable(varnum: Cardinal; value: String)');
  Comp.AddDelphiFunction('function MessageBox(const Text, Caption: String; uType: Cardinal): Integer;');
  Comp.AddDelphiFunction('function ParamStr(I: Integer): String)');
  Comp.AddDelphiFunction('procedure DetailPrint(const S: String)');
  Comp.AddDelphiFunction('function GetStackTop: Pointer');
  Comp.AddDelphiFunction('function ParamCount: Integer');
  Comp.AddDelphiFunction('function CallInstDll(const DLLFileName, FuncName: String; const NoUnload: Boolean): Boolean');
end;

end.

